@@ -19,8 +19,8 @@
  *
  *
  *  Notes:
- *    Assumes any number of processors, cores and fans (max. 6, 16, 12
- *    as defined in NUM_PROCESSORS, NUM_HWMONS, NUM_TEMP_INPUTS and NUM_FANS)
+ *    Assumes any number of processors, cores, sensors and fans
+ *    (as defined in NUM_PROCESSORS, NUM_HWMONS, NUM_TEMP_INPUTS and NUM_FANS)
  *    It uses only the temperatures from the processors as input.
  *    Requires coretemp and applesmc kernel modules to be loaded.
  *    Requires root use
@@ -42,6 +42,7 @@
 #include "mbpfan.h"
 #include "global.h"
 #include "settings.h"
+#include "util.h"
 
 /* lazy min/max... */
 #define min(a,b) ((a) < (b) ? (a) : (b))
@@ -58,7 +59,7 @@ int max_temp = 86;   // do not set it > 90
 // maximum number of processors etc supported
 #define NUM_PROCESSORS 6
 #define NUM_HWMONS 12
-#define NUM_TEMP_INPUTS 16
+#define NUM_TEMP_INPUTS 64
 #define NUM_FANS 10
 // sane defaults when user provides unexpected values
 #define MIN_FAN_SPEED_DEFAULT 500
@@ -100,8 +101,7 @@ bool is_modern_sensors_path()
     str_kernel_version = strtok(kernel.release, ".");
 
     if (atoi(str_kernel_version) < 3){
-        syslog(LOG_INFO, "mbpfan detected a pre-3.x.x linux kernel. Detected version: %s. Exiting.\n", kernel.release);
-        printf("mbpfan detected a pre-3.x.x linux kernel. Detected version: %s. Exiting.\n", kernel.release);
+        mbp_log(LOG_ERR, "mbpfan detected a pre-3.x.x linux kernel. Detected version: %s. Exiting.\n", kernel.release);
         exit(EXIT_FAILURE);
     }
 
@@ -136,11 +136,7 @@ t_sensors *retrieve_sensors()
 
     if (!is_modern_sensors_path()) {
         if(verbose) {
-            printf("Using legacy sensor path for kernel < 3.15.0\n");
-
-            if(daemonize) {
-                syslog(LOG_INFO, "Using legacy path for kernel < 3.15.0");
-            }
+            mbp_log(LOG_INFO, "Using legacy path for kernel < 3.15.0");
         }
 
         path_begin = strdup("/sys/devices/platform/coretemp.0/temp");
@@ -148,11 +144,7 @@ t_sensors *retrieve_sensors()
     } else {
 
         if(verbose) {
-            printf("Using new sensor path for kernel >= 3.15.0 or some CentOS versions with kernel 3.10.0\n");
-
-            if(daemonize) {
-                syslog(LOG_INFO, "Using new sensor path for kernel >= 3.15.0 or some CentOS versions with kernel 3.10.0 ");
-            }
+            mbp_log(LOG_INFO, "Using new sensor path for kernel >= 3.15.0 or some CentOS versions with kernel 3.10.0 ");
         }
 
 	// loop over up to 6 processors
@@ -167,9 +159,7 @@ t_sensors *retrieve_sensors()
 	    int counter;
 	    for (counter = 0; counter < NUM_HWMONS; counter++) {
 
-		char hwmon_path[strlen(path_begin)+2];
-
-		sprintf(hwmon_path, "%s%d", path_begin, counter);
+                char *hwmon_path = smprintf("%s%d", path_begin, counter);
 
 		int res = access(hwmon_path, R_OK);
 		if (res == 0) {
@@ -178,16 +168,14 @@ t_sensors *retrieve_sensors()
 		    path_begin = smprintf("%s/temp", hwmon_path);
 
 		    if(verbose) {
-			printf("Found hwmon path at %s\n", path_begin);
-
-			if(daemonize) {
-			    syslog(LOG_INFO, "Found hwmon path at %s\n", path_begin);
-			}
-
+                        mbp_log(LOG_INFO, "Found hwmon path at %s", path_begin);
 		    }
 
+                    free(hwmon_path);
 		    break;
 		}
+
+                free(hwmon_path);
 	    }
 
 	    int core = 0;
@@ -227,16 +215,11 @@ t_sensors *retrieve_sensors()
     }
 
     if(verbose) {
-        printf("Found %d sensors\n", sensors_found);
-
-        if(daemonize) {
-            syslog(LOG_INFO, "Found %d sensors", sensors_found);
-        }
+        mbp_log(LOG_INFO, "Found %d sensors", sensors_found);
     }
 
     if (sensors_found == 0){
-        syslog(LOG_CRIT, "mbpfan could not detect any temp sensor. Please contact the developer.\n");
-        printf("mbpfan could not detect any temp sensor. Please contact the developer.\n");
+        mbp_log(LOG_CRIT, "mbpfan could not detect any temp sensor. Please contact the developer.");
         exit(EXIT_FAILURE);
     }
 
@@ -257,18 +240,38 @@ static int read_value(const char *path)
     return value;
 }
 
+static void read_value_str(const char *path, char *str, size_t len)
+{
+    FILE *file = fopen(path, "r");
+    if (file != NULL) {
+        fgets(str, len, file);
+        fclose(file);
+    }
+}
+
+static void trim_trailing_whitespace(char *str)
+{
+    for (ssize_t i = strlen(str) - 1; i >= 0; --i) {
+        if (isspace(str[i]) || str[i] == '\n') {
+            str[i] = '\0';
+        }
+    }
+}
+
 t_fans *retrieve_fans()
 {
     t_fans *fans_head = NULL;
     t_fans *fan = NULL;
 
     char *path_output = NULL;
+    char *path_label = NULL;
     char *path_manual = NULL;
     char *path_fan_max = NULL;
     char *path_fan_min = NULL;
 
     const char *path_begin = "/sys/devices/platform/applesmc.768/fan";
     const char *path_output_end = "_output";
+    const char *path_label_end = "_label";
     const char *path_man_end = "_manual";
     const char *path_max_speed = "_max";
     const char *path_min_speed = "_min";
@@ -279,6 +282,7 @@ t_fans *retrieve_fans()
     for(counter = 0; counter<NUM_FANS; counter++) {
 
         path_output = smprintf("%s%d%s", path_begin, counter, path_output_end);
+        path_label = smprintf("%s%d%s", path_begin, counter, path_label_end);
         path_manual = smprintf("%s%d%s", path_begin, counter, path_man_end);
 	path_fan_min = smprintf("%s%d%s",path_begin, counter, path_min_speed);
 	path_fan_max = smprintf("%s%d%s",path_begin, counter, path_max_speed);
@@ -302,7 +306,12 @@ t_fans *retrieve_fans()
 		fan->fan_max_speed = MAX_FAN_SPEED_DEFAULT;
 	    else
 		fan->fan_max_speed = fan_speed;
-	    
+
+            size_t max_label_len = 64;
+            fan->label = malloc(max_label_len);
+            read_value_str(path_label, fan->label, max_label_len);
+            trim_trailing_whitespace(fan->label);
+
             fan->old_speed = 0;
 
             if (fans_head == NULL) {
@@ -325,6 +334,8 @@ t_fans *retrieve_fans()
         }
 	free(path_fan_min);
 	path_fan_min = NULL;
+	free(path_label);
+	path_label = NULL;
 	free(path_fan_max);
 	path_fan_max = NULL;
         free(path_output);
@@ -334,16 +345,11 @@ t_fans *retrieve_fans()
     }
 
     if(verbose) {
-        printf("Found %d fans\n", fans_found);
-
-        if(daemonize) {
-            syslog(LOG_INFO, "Found %d fans", fans_found);
-        }
+        mbp_log(LOG_INFO, "Found %d fans", fans_found);
     }
 
     if (fans_found == 0){
-        syslog(LOG_CRIT, "mbpfan could not detect any fan. Please contact the developer.\n");
-        printf("mbpfan could not detect any fan. Please contact the developer.\n");
+        mbp_log(LOG_CRIT, "mbpfan could not detect any fan. Please contact the developer.");
         exit(EXIT_FAILURE);
     }
 
@@ -388,7 +394,7 @@ t_sensors *refresh_sensors(t_sensors *sensors)
             char buf[16];
             int len = pread(fileno(tmp->file), buf, sizeof(buf), /*offset=*/ 0);
             buf[len] = '\0';
-            sscanf(buf, "%d", &tmp->temperature);
+            tmp->temperature = strtod(buf, NULL);
         }
 
         tmp = tmp->next;
@@ -423,26 +429,16 @@ void set_fan_minimum_speed(t_fans* fans)
 unsigned short get_temp(t_sensors* sensors)
 {
     sensors = refresh_sensors(sensors);
-    int sum_temp = 0;
-    unsigned short temp = 0;
+    unsigned int temp = 0;
 
     t_sensors* tmp = sensors;
 
-    int number_sensors = 0;
-
     while(tmp != NULL) {
-        sum_temp += tmp->temperature;
+        temp = max(temp, tmp->temperature);
         tmp = tmp->next;
-        number_sensors++;
     }
 
-    // just to be safe
-    if (number_sensors == 0) {
-        number_sensors++;
-    }
-
-    temp = (unsigned short)( ceil( (float)( sum_temp ) / (number_sensors * 1000) ) );
-    return temp;
+    return temp / 1000;
 }
 
 void retrieve_settings(const char* settings_path, t_fans* fans)
@@ -462,11 +458,7 @@ void retrieve_settings(const char* settings_path, t_fans* fans)
     if (f == NULL) {
         /* Could not open configfile */
         if(verbose) {
-            printf("Couldn't open configfile, using defaults\n");
-
-            if(daemonize) {
-                syslog(LOG_INFO, "Couldn't open configfile, using defaults");
-            }
+            mbp_log(LOG_INFO, "Couldn't open configfile, using defaults");
         }
 
     } else {
@@ -476,11 +468,7 @@ void retrieve_settings(const char* settings_path, t_fans* fans)
         if (settings == NULL) {
             /* Could not read configfile */
             if(verbose) {
-                printf("Couldn't read configfile\n");
-
-                if(daemonize) {
-                    syslog(LOG_WARNING, "Couldn't read configfile");
-                }
+                mbp_log(LOG_WARNING, "Couldn't read configfile");
             }
 
         } else {
@@ -551,16 +539,14 @@ void mbpfan()
     while(fan != NULL) {
 	
         if (fan->fan_min_speed > fan->fan_max_speed) {
-            syslog(LOG_INFO, "Invalid fan speeds: %d %d", fan->fan_min_speed,  fan->fan_max_speed);
-            printf("Invalid fan speeds: %d %d\n", fan->fan_min_speed, fan->fan_max_speed);
+            mbp_log(LOG_ERR, "Invalid fan speeds: %d %d", fan->fan_min_speed,  fan->fan_max_speed);
             exit(EXIT_FAILURE);
         }
 	fan = fan->next;
     }
 
     if (low_temp > high_temp || high_temp > max_temp) {
-        syslog(LOG_INFO, "Invalid temperatures: %d %d %d", low_temp, high_temp, max_temp);
-        printf("Invalid temperatures: %d %d %d\n", low_temp, high_temp, max_temp);
+        mbp_log(LOG_ERR, "Invalid temperatures: %d %d %d", low_temp, high_temp, max_temp);
         exit(EXIT_FAILURE);
     }
 
@@ -573,19 +559,16 @@ void mbpfan()
     while(fan != NULL) {
 
        fan->step_up = (float)( fan->fan_max_speed - fan->fan_min_speed ) /
-                      (float)( ( max_temp - high_temp ) * ( max_temp - high_temp + 1 ) / 2 );
+                      (float)( ( max_temp - high_temp ) * ( max_temp - high_temp + 1 ) / 2.0 );
 
        fan->step_down = (float)( fan->fan_max_speed - fan->fan_min_speed ) /
-                        (float)( ( max_temp - low_temp ) * ( max_temp - low_temp + 1 ) / 2 );
+                        (float)( ( max_temp - low_temp ) * ( max_temp - low_temp + 1 ) / 2.0 );
        fan = fan->next;
     }
 
+recalibrate:
     if(verbose) {
-        printf("Sleeping for 2 seconds to get first temp delta\n");
-
-        if(daemonize) {
-            syslog(LOG_INFO, "Sleeping for 2 seconds to get first temp delta");
-        }
+        mbp_log(LOG_INFO, "Sleeping for 2 seconds to get first temp delta");
     }
     sleep(2);
 
@@ -619,11 +602,7 @@ void mbpfan()
             }
 
             if(verbose) {
-                printf("Old Temp %d: New Temp: %d, Fan Speed: %d\n", old_temp, new_temp, fan_speed);
-	    
-                if(daemonize) {
-                   syslog(LOG_INFO, "Old Temp %d: New Temp: %d, Fan Speed: %d", old_temp, new_temp, fan_speed);
-                }
+                mbp_log(LOG_INFO, "Old Temp: %d New Temp: %d Fan: %s Speed: %d", old_temp, new_temp, fan->label, fan_speed);
 	    }
 
 	    set_fan_speed(fan, fan_speed);
@@ -631,19 +610,23 @@ void mbpfan()
 	} 
 
         if(verbose) {
-            printf("Sleeping for %d seconds\n", polling_interval);
-            fflush(stdout);
-
-            if(daemonize) {
-                syslog(LOG_INFO, "Sleeping for %d seconds", polling_interval);
-            }
+            mbp_log(LOG_INFO, "Sleeping for %d seconds", polling_interval);
         }
    
+        time_t before_sleep = time(NULL);
+
         // call nanosleep instead of sleep to avoid rt_sigprocmask and
         // rt_sigaction
         struct timespec ts;
         ts.tv_sec = polling_interval;
         ts.tv_nsec = 0;
         nanosleep(&ts, NULL);
+
+        time_t after_sleep = time(NULL);
+        if(after_sleep - before_sleep > 2 * polling_interval) {
+            mbp_log(LOG_INFO, "Clock skew detected - slept for %ld seconds but expected %d", after_sleep - before_sleep, polling_interval);
+            set_fans_man(fans);
+            goto recalibrate;
+        }
     }
 }
