@@ -17,6 +17,8 @@
 #ifndef _MBPFAN_H_
 #define _MBPFAN_H_
 
+#include <stdbool.h>
+
 /** Temperature Thresholds
  *  low_temp - temperature below which fan speed will be at minimum
  *  high_temp - fan will increase speed when higher than this temperature
@@ -94,7 +96,7 @@ void set_fan_speed(t_fans* fan, int speed);
  */
 void set_fan_minimum_speed(t_fans* fans);
 /**
- *  Return average CPU temp in degrees (ceiling)
+ *  Return maximum CPU temp in degrees
  */
 unsigned short get_temp(t_sensors* sensors);
 
